@@ -6,7 +6,7 @@
 # @License: MIT License
 
 import os
-from typing import List
+from typing import List, Callable
 
 import numpy as np
 import pandas as pd
@@ -76,6 +76,25 @@ class BaderStartZero(_BasePathOut):
         else:
             return None
 
+    @staticmethod
+    def extract(data: pd.DataFrame, atoms, format_path: Callable = None):
+
+        if isinstance(atoms, (list, tuple)):
+            data = data.apply(pd.to_numeric, errors='ignore')
+            res = []
+            if format_path is not None:
+                data["File"] = [format_path(ci) for ci in data["File"]]
+            for v in atoms:
+                sel = data[data["Atom Number"] == v + 1]
+                sel = sel[["File", "Bader Ele Move"]].set_index("File")
+                n_name = [f"{i}-{v}" for i in sel.columns]
+                sel.columns = n_name
+                res.append(sel)
+            return pd.concat(res, axis=1)
+        else:
+            raise NotImplementedError
+
+
     def run(self, path: Path, files: List = None):
         """3.Run with software and necessary file and get data.
         (1) Return result in code, or (2) Both return file to each path and in code."""
